#!/usr/bin/perl -W

# combined extract varname / extract comment script

# usage: 'get_name.pl global.h'

if (@ARGV != 4) {
    print "usage:\n";
    print "\t get_name global.h varstrname.h magicstrname.h configstrname\n\n";
    exit;
}

my ($headerfilename, $varstrname, $magicstrname, $configstrname ) = @ARGV;

$struct_found = 0;
$cbyte_count = 0;

	$nameID = "_varname_str_";
	$commentID = "_magic_str_";
	$varstr = "varstr[]";
	$magicstr = "magicstr[]";

if ($configstrname eq 'config_ext_t') {
	$nameID = "_varname_ext_str_";
	$commentID = "_magic_ext_str_";
	$varstr = "varstr_ext[]";
	$magicstr = "magicstr_ext[]";
}

if ($configstrname eq 'config_shared_t') {
	$nameID = "_varname_shared_str_";
	$commentID = "_magic_shared_str_";
	$varstr = "varstr_shared[]";
	$magicstr = "magicstr_shared[]";
}


open NAME, ">$varstrname";
print NAME "\n// Automatically generated file (bin/get_name.pl), DO NOT EDIT directly\n\n";

open COMMENT, ">$magicstrname";
print COMMENT "\n// Automatically generated file (bin/get_name.pl), DO NOT EDIT directly\n\n";


while($l = <>) {
	chomp $l;

	if ($l =~ /^\s*($|\/\/)/) {  # skip empty lines or lines consisting of comments only
	next;
    }
    
    if ($l =~ /^\s*struct $configstrname\s*{/) { # find config_t structure
	$struct_found = 1;
    } elsif ($l =~ /\s*};/) {
	$struct_found = 0;
    } elsif ($struct_found == 1) { # examine structure

	# limited to 48 characters per displayed string
	($str) = ($l =~ /\s*\/\/\s*(.{1,70})/);	# comments after 2nd // ignored : (\s*\/\/)?
	
	my $hex_cbyte_count = sprintf("%02x", $cbyte_count);
	$magicNname = $nameID . $hex_cbyte_count;
	$magicCname = $commentID . $hex_cbyte_count;

	if ($l =~ /uint8_t ([\[\]\w]+)\s*;/) {  # check if the line contains something we know

	    if ($1 =~ /(\w+)\[(\d+)\]/) {  # found an array "test[]"
		$varname = $1;
		$varsize = $2;

		print NAME "const char $magicNname\[\] PROGMEM = \"$varname\";\n\n";
		print COMMENT "const char $magicCname\[\] PROGMEM = \"$str\";\n\n";
		$Carr .= $magicCname .",\n";
		$Narr .= $magicNname .",\n";

		for ($i=1; $i<$varsize; $i++) {
		    # a little space consuming (but space saving on the simple program...)
		    # the previous non-NULL pointer will need to be searched
		    $Carr .= "NULL,";
		    $Narr .= "NULL,";
		}
		$Carr .= "\n";
		$Narr .= "\n";

	    } else {  # found a variable "test"
		$varname = $1;
		$varsize = 1;

		print NAME "const char $magicNname\[\] PROGMEM = \"$varname\";\n";
		print COMMENT "const char $magicCname\[\] PROGMEM = \"$str\";\n";
		$Carr .= $magicCname .",\n";
		$Narr .= $magicNname .",\n";
	    }

	    $cbyte_count += $varsize;

	} else {
	    print "Ooops, can't understand \"$l\"\n";
	}
    }
}

# ----------- Done with global.h parsing ---------

print NAME "\nPGM_VOID_P const $varstr PROGMEM = {\n $Narr NULL\n};\n";	# with the unnecessary NULL we just spare some coding for the last ','
close NAME;

# we pad variables to multiple of 16, so avr does not display silly things
# at the end (but we can still write to silly places - the igntable)
$i = 16 - (($cbyte_count) % 16);

$magicCname = $commentID . $cbyte_count;
if($i > 0){
    print COMMENT "const char $magicCname\[\] PROGMEM = \"No more variable, go back, don't touch\";\n";
		$Carr .= $magicCname .",\n";
    $i--;
}

# if $i is still more than 0, we padd NULL pointers
while($i > 0){
    $Carr .= "NULL,";			# the previous non-NULL pointer will need to be searched
    $i--;
}

print COMMENT "\nPGM_VOID_P const $magicstr PROGMEM = {\n $Carr \nNULL\n};\n";	# with the unnecessary NULL we just spare some coding for the last ','
close COMMENT;
